import re

from django.conf import settings
from django.contrib.sites.models import Site
from django.core import urlresolvers
from django.db.utils import DatabaseError
import django.conf.urls as urls

from maintenancemode.models import Maintenance, IgnoredURL
from maintenancemode.utils.settings import DJANGO_MINOR_VERSION

urls.handler503 = 'maintenancemode.views.defaults.temporary_unavailable'
urls.__all__.append('handler503')


class MaintenanceModeMiddleware(object):
    def process_request(self, request):
        """
        Get the maintenance mode from the database.
        If a Maintenance value doesn't already exist in the database, we'll create one.
        "has_add_permission" and "has_delete_permission" are overridden in admin
        to prevent the user from adding or deleting a record, as we only need one
        to affect multiple sites managed from one instance of Django admin.
        """
        site = Site.objects.get_current()

        try:
            maintenance = Maintenance.objects.get(site=site)
        except (Maintenance.DoesNotExist, DatabaseError):
            for site in Site.objects.all():
                maintenance = Maintenance.objects.create(site=site, is_being_performed=False)

        # Allow access if maintenance is not being performed
        if not maintenance.is_being_performed:
            return None

        # Allow access if remote ip is in INTERNAL_IPS
        if request.META.get('REMOTE_ADDR') in settings.INTERNAL_IPS:
            return None

        # Allow access if the user doing the request is logged in and a
        # staff member.
        if hasattr(request, 'user') and request.user.is_staff:
            return None

        # Check if a path is explicitly excluded from maintenance mode
        urls_to_ignore = IgnoredURL.objects.filter(maintenance=maintenance)
        ignore_urls = tuple([re.compile(r'%s' % str(url.pattern)) for url in urls_to_ignore])
        request_path = request.path_info[1:]
        for url in ignore_urls:
            if url.match(request_path):
                return None

        # Otherwise show the user the 503 page
        resolver = urlresolvers.get_resolver(None)

        if DJANGO_MINOR_VERSION < 8:
            callback, param_dict = resolver._resolve_special('503')
        else:
            callback, param_dict = resolver.resolve_error_handler('503')

        return callback(request, **param_dict)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):

        # Adding model 'Maintenance'
        db.create_table('maintenancemode_maintenance', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('site', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['sites.Site'])),
            ('is_being_performed', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('maintenancemode', ['Maintenance'])

        # Adding model 'IgnoredURL'
        db.create_table('maintenancemode_ignoredurl', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('maintenance', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['maintenancemode.Maintenance'])),
            ('pattern', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('description', self.gf('django.db.models.fields.CharField')(max_length=75)),
        ))
        db.send_create_signal('maintenancemode', ['IgnoredURL'])


    def backwards(self, orm):

        # Deleting model 'Maintenance'
        db.delete_table('maintenancemode_maintenance')

        # Deleting model 'IgnoredURL'
        db.delete_table('maintenancemode_ignoredurl')


    models = {
        'maintenancemode.ignoredurl': {
            'Meta': {'object_name': 'IgnoredURL'},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '75'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'maintenance': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['maintenancemode.Maintenance']"}),
            'pattern': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'maintenancemode.maintenance': {
            'Meta': {'object_name': 'Maintenance'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_being_performed': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"})
        },
        'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['maintenancemode']



from django.contrib import admin

from maintenancemode.models import Maintenance, IgnoredURL


class IgnoredURLInline(admin.TabularInline):
    model = IgnoredURL
    extra = 3


class MaintenanceAdmin(admin.ModelAdmin):
    inlines = [IgnoredURLInline, ]
    list_display = ['__unicode__', 'is_being_performed']
    readonly_fields = ('site',)
    actions = None

    def has_delete_permission(self, request, obj=None):
        return False

    def has_add_permission(self, request):
        return False

admin.site.register(Maintenance, MaintenanceAdmin)



from django.template import Context, loader, RequestContext

from maintenancemode.http import HttpResponseTemporaryUnavailable

def temporary_unavailable(request, template_name='503.html'):
    """
    Default 503 handler, which looks for the requested URL in the redirects
    table, redirects if found, and displays 404 page if not redirected.
    
    Templates: `503.html`
    Context:
        request_path
            The path of the requested URL (e.g., '/app/pages/bad_page/')
    """
    return HttpResponseTemporaryUnavailable(loader.render_to_string(template_name, {
        'request_path': request.path,
    }, RequestContext(request)))



from django.contrib.sites.models import Site
from django.db import models
from django.db.models.signals import post_save, pre_save, pre_delete
from django.template.defaultfilters import join


class Maintenance(models.Model):
    site = models.ForeignKey(Site)
    is_being_performed = models.BooleanField('In Maintenance Mode', default=False)

    class Meta:
        verbose_name = verbose_name_plural = 'Maintenance Mode'

    def __unicode__(self):
        return self.site.domain

class IgnoredURL(models.Model):
    maintenance = models.ForeignKey(Maintenance)
    pattern = models.CharField(max_length=255)
    description = models.CharField(max_length=75, help_text='What this URL pattern covers.')

    def __unicode__(self):
        return self.pattern



import posixpath

from django.core.exceptions import ImproperlyConfigured

from maintenancemode.utils.settings import AppSettings


class MaintenanceModeSettings(AppSettings):
    CACHE_BACKEND = None

    def configure_cache_backend(self, value):
        # If we are on Django >= 1.3 AND using the new CACHES setting...
        if hasattr(self, "CACHES"):
            if "maintenance_mode" in self.CACHES:
                return "maintenance_mode"
            else:
                return "default"
        if isinstance(value, basestring) and value.startswith("maintenance_mode."):
            raise ImproperlyConfigured("Please upgrade to one of the supported backends "
                                       "for the version of Django you are using as defined in the "
                                       "Django docs: https://docs.djangoproject.com/en/dev/topics/cache/")
        return value

settings = MaintenanceModeSettings("MAINTENANCE_MODE")
